@@ -38,8 +38,12 @@
 #include <spice/protocol.h>
 #include <spice/macros.h>
 
-#ifdef USE_SMARTCARD
+#ifdef USE_SMARTCARD_012
 #include <vscard_common.h>
+#else
+#ifdef USE_SMARTCARD
+#include <libcacard.h>
+#endif
 #endif
 
 #include "draw.h"
@@ -51,6 +55,13 @@ typedef struct SpiceMsgData {
     uint8_t data[0];
 } SpiceMsgData;
 
+typedef struct SpiceMsgCompressedData {
+    uint8_t type;
+    uint32_t uncompressed_size;
+    uint32_t compressed_size;
+    uint8_t *compressed_data;
+} SpiceMsgCompressedData;
+
 typedef struct SpiceMsgEmpty {
     uint8_t padding;
 } SpiceMsgEmpty;
@@ -67,7 +78,7 @@ typedef struct SpiceMsgMainMultiMediaTime {
     uint32_t time;
 } SpiceMsgMainMultiMediaTime;
 
-typedef struct SpiceMsgMainMigrationBegin {
+typedef struct SpiceMigrationDstInfo {
     uint16_t port;
     uint16_t sport;
     uint32_t host_size;
@@ -77,8 +88,21 @@ typedef struct SpiceMsgMainMigrationBegin {
     uint8_t *pub_key_data;
     uint32_t cert_subject_size;
     uint8_t *cert_subject_data;
+} SpiceMigrationDstInfo;
+
+typedef struct SpiceMsgMainMigrationBegin {
+    SpiceMigrationDstInfo dst_info;
 } SpiceMsgMainMigrationBegin;
 
+typedef struct SpiceMsgMainMigrateBeginSeamless {
+    SpiceMigrationDstInfo dst_info;
+    uint32_t src_mig_version;
+} SpiceMsgMainMigrateBeginSeamless;
+
+typedef struct SpiceMsgcMainMigrateDstDoSeamless {
+    uint32_t src_version;
+} SpiceMsgcMainMigrateDstDoSeamless;
+
 typedef struct SpiceMsgMainMigrationSwitchHost {
     uint16_t port;
     uint16_t sport;
@@ -189,6 +213,8 @@ typedef struct SpiceMsgMainAgentTokens {
     uint32_t num_tokens;
 } SpiceMsgMainAgentTokens, SpiceMsgcMainAgentTokens, SpiceMsgcMainAgentStart;
 
+typedef struct SpiceMsgMainAgentTokens SpiceMsgMainAgentConnectedTokens;
+
 typedef struct SpiceMsgcClientInfo {
     uint64_t cache_size;
 } SpiceMsgcClientInfo;
@@ -253,6 +279,11 @@ typedef struct SpiceMsgDisplayDrawAlphaBlend {
     SpiceAlphaBlend data;
 } SpiceMsgDisplayDrawAlphaBlend;
 
+typedef struct SpiceMsgDisplayDrawComposite {
+    SpiceMsgDisplayBase base;
+    SpiceComposite data;
+} SpiceMsgDisplayDrawComposite;
+
 typedef struct SpiceMsgDisplayCopyBits {
     SpiceMsgDisplayBase base;
     SpicePoint src_pos;
@@ -296,7 +327,7 @@ typedef struct SpiceMsgDisplayInvalOne {
 
 typedef struct SpiceMsgDisplayStreamCreate {
     uint32_t surface_id;
-    uint32_t id;
+    uint32_t id; /* Any number from 0 to SPICE_MAX_NUM_STREAMS - 1 */
     uint32_t flags;
     uint32_t codec_type;
     uint64_t stamp;
@@ -337,6 +368,27 @@ typedef struct SpiceMsgDisplayStreamDestroy {
     uint32_t id;
 } SpiceMsgDisplayStreamDestroy;
 
+typedef struct SpiceMsgDisplayStreamActivateReport {
+    uint32_t stream_id;
+    uint32_t unique_id;
+    uint32_t max_window_size;
+    uint32_t timeout_ms;
+} SpiceMsgDisplayStreamActivateReport;
+
+typedef struct SpiceMsgcDisplayStreamReport {
+    uint32_t stream_id;
+    uint32_t unique_id;
+    uint32_t start_frame_mm_time;
+    uint32_t end_frame_mm_time;
+    uint32_t num_frames;
+    uint32_t num_drops;
+    int32_t last_frame_delay;
+    uint32_t audio_delay;
+} SpiceMsgcDisplayStreamReport;
+
+typedef struct SpiceMsgcDisplayGlDrawDone {
+} SpiceMsgcDisplayGlDrawDone;
+
 typedef struct SpiceMsgCursorInit {
     SpicePoint16 position;
     uint16_t trail_length;
@@ -431,6 +483,10 @@ typedef struct SpiceMsgPlaybackPacket {
     uint32_t data_size;
 } SpiceMsgPlaybackPacket, SpiceMsgcRecordPacket;
 
+typedef struct SpiceMsgPlaybackLatency {
+    uint32_t latency_ms;
+} SpiceMsgPlaybackLatency;
+
 typedef struct SpiceMsgRecordStart {
     uint32_t channels;
     uint32_t format; //SPICE_AUDIO_FMT_?
@@ -574,6 +630,45 @@ typedef struct SpiceMsgDisplayMonitorsConfig {
     SpiceHead heads[0];
 } SpiceMsgDisplayMonitorsConfig;
 
+typedef struct SpiceMsgPortInit {
+    uint32_t name_size;
+    uint8_t *name;
+    uint8_t opened;
+} SpiceMsgPortInit;
+
+typedef struct SpiceMsgPortEvent {
+    uint8_t event;
+} SpiceMsgPortEvent;
+
+typedef struct SpiceMsgcPortEvent {
+    uint8_t event;
+} SpiceMsgcPortEvent;
+
+typedef struct SpiceMsgcDisplayPreferredVideoCodecType {
+    uint8_t num_of_codecs;
+    uint8_t codecs[0];
+} SpiceMsgcDisplayPreferredVideoCodecType;
+
+typedef struct SpiceMsgcDisplayPreferredCompression {
+    uint8_t image_compression;
+} SpiceMsgcDisplayPreferredCompression;
+
+typedef struct SpiceMsgDisplayGlScanoutUnix {
+    int drm_dma_buf_fd;
+    uint32_t width;
+    uint32_t height;
+    uint32_t stride;
+    uint32_t drm_fourcc_format;
+    uint32_t flags;
+} SpiceMsgDisplayGlScanoutUnix;
+
+typedef struct SpiceMsgDisplayGlDraw {
+    uint32_t x;
+    uint32_t y;
+    uint32_t w;
+    uint32_t h;
+} SpiceMsgDisplayGlDraw;
+
 SPICE_END_DECLS
 
 #endif /* _H_SPICE_PROTOCOL */
