@@ -36,23 +36,39 @@ G_BEGIN_DECLS
 
 typedef struct display_stream display_stream;
 
+typedef struct SpiceFrame SpiceFrame;
+struct SpiceFrame {
+    uint32_t mm_time;
+    SpiceRect dest;
+
+    uint8_t *data;
+    uint32_t size;
+    gpointer data_opaque;
+    void (*ref_data)(gpointer data_opaque);
+    void (*unref_data)(gpointer data_opaque);
+
+    void (*free)(SpiceFrame *frame);
+};
+
 typedef struct VideoDecoder VideoDecoder;
 struct VideoDecoder {
     /* Releases the video decoder's resources */
     void (*destroy)(VideoDecoder *decoder);
 
     /* Notifies the decoder that the mm-time clock changed. */
-    void (*reschedule)(VideoDecoder *video_decoder);
+    void (*reschedule)(VideoDecoder *decoder);
 
-    /* Decompresses the specified frame.
+    /* Takes ownership of the specified frame, decompresses it,
+     * and displays it at the right time.
      *
      * @decoder:   The video decoder.
-     * @frame_msg: The Spice message containing the compressed frame.
-     * @return:    A pointer to the buffer holding the decoded frame. This
-     *             buffer will be invalidated by the next call to
-     *             decode_frame().
+     * @frame:     The compressed Spice frame.
+     * @latency:   How long in milliseconds until the frame should be
+     *             displayed. Negative values mean the frame is late.
+     * @return:    False if the decoder can no longer decode frames,
+     *             True otherwise.
      */
-    void (*queue_frame)(VideoDecoder *decoder, SpiceMsgIn *frame_msg, int32_t latency);
+    gboolean (*queue_frame)(VideoDecoder *video_decoder, SpiceFrame *frame, int latency);
 
     /* The format of the encoded video. */
     int codec_type;
@@ -82,6 +98,7 @@ gboolean gstvideo_has_codec(int codec_type);
 typedef struct display_surface {
     guint32                     surface_id;
     bool                        primary;
+    bool                        streaming_mode;
     enum SpiceSurfaceFmt        format;
     int                         width, height, stride, size;
     uint8_t                     *data;
@@ -98,12 +115,12 @@ typedef struct drops_sequence_stats {
 } drops_sequence_stats;
 
 struct display_stream {
-    SpiceMsgIn                  *msg_create;
-    SpiceMsgIn                  *msg_clip;
-
     /* from messages */
+    uint32_t                    id;
+    uint32_t                    flags;
+    SpiceRect                   dest;
     display_surface             *surface;
-    const SpiceClip             *clip;
+    SpiceClip                   clip;
     QRegion                     region;
     int                         have_region;
 
@@ -135,10 +152,51 @@ struct display_stream {
     uint32_t report_drops_seq_len;
 };
 
+static const struct {
+    int cap;
+    const gchar name[8];
+
+    /* FIXME: With gstreamer version 1.9.0 and higher, we are using playbin to
+     * create the pipeline for us and for that reason we don't need to keep track of
+     * decoder's name anymore. */
+    const gchar *dec_name;
+    const gchar *dec_caps;
+} gst_opts[] = {
+    /* SpiceVideoCodecType starts at index 1 */
+    { 0 },
+
+    /* SPICE_VIDEO_CODEC_TYPE_MJPEG */
+    { SPICE_DISPLAY_CAP_CODEC_MJPEG, "mjpeg",
+      "jpegdec", "image/jpeg" },
+
+    /* SPICE_VIDEO_CODEC_TYPE_VP8
+     *
+     * typefind is unable to identify VP8 streams by design.
+     * See: https://bugzilla.gnome.org/show_bug.cgi?id=756457
+     */
+    { SPICE_DISPLAY_CAP_CODEC_VP8, "vp8",
+      "vp8dec", "video/x-vp8" },
+
+    /* SPICE_VIDEO_CODEC_TYPE_H264
+     * When setting video/x-h264, h264parse will complain if we don't have the
+     * stream-format or codec_data information. As stream-format is byte-stream
+     * (hardcoded in spice-server), let's add it here to avoid the warning.
+     */
+    { SPICE_DISPLAY_CAP_CODEC_H264, "h264",
+      "h264parse ! avdec_h264", "video/x-h264" },
+
+    /* SPICE_VIDEO_CODEC_TYPE_VP9 */
+    { SPICE_DISPLAY_CAP_CODEC_VP9, "vp9",
+      "vp9dec", "video/x-vp9" },
+};
+
+G_STATIC_ASSERT(G_N_ELEMENTS(gst_opts) <= SPICE_VIDEO_CODEC_TYPE_ENUM_END);
+
 guint32 stream_get_time(display_stream *st);
 void stream_dropped_frame_on_playback(display_stream *st);
-void stream_display_frame(display_stream *st, SpiceMsgIn *frame_msg, uint32_t width, uint32_t height, uint8_t *data);
-uint32_t spice_msg_in_frame_data(SpiceMsgIn *frame_msg, uint8_t **data);
+#define SPICE_UNKNOWN_STRIDE 0
+void stream_display_frame(display_stream *st, SpiceFrame *frame, uint32_t width, uint32_t height, int stride, uint8_t* data);
+guintptr get_window_handle(display_stream *st);
 
 
 G_END_DECLS
