@@ -49,9 +49,6 @@
  * #SpiceDisplayChannel::display-invalidate signals.
  */
 
-#define SPICE_DISPLAY_CHANNEL_GET_PRIVATE(obj)                                  \
-    (G_TYPE_INSTANCE_GET_PRIVATE((obj), SPICE_TYPE_DISPLAY_CHANNEL, SpiceDisplayChannelPrivate))
-
 #define MONITORS_MAX 256
 
 struct _SpiceDisplayChannelPrivate {
@@ -70,13 +67,10 @@ struct _SpiceDisplayChannelPrivate {
     GArray                      *monitors;
     guint                       monitors_max;
     gboolean                    enable_adaptive_streaming;
-#ifdef G_OS_WIN32
-    HDC dc;
-#endif
     SpiceGlScanout scanout;
 };
 
-G_DEFINE_TYPE(SpiceDisplayChannel, spice_display_channel, SPICE_TYPE_CHANNEL)
+G_DEFINE_TYPE_WITH_PRIVATE(SpiceDisplayChannel, spice_display_channel, SPICE_TYPE_CHANNEL)
 
 /* Properties */
 enum {
@@ -94,6 +88,7 @@ enum {
     SPICE_DISPLAY_INVALIDATE,
     SPICE_DISPLAY_MARK,
     SPICE_DISPLAY_GL_DRAW,
+    SPICE_DISPLAY_STREAMING_MODE,
 
     SPICE_DISPLAY_LAST_SIGNAL,
 };
@@ -109,6 +104,7 @@ static display_surface *find_surface(SpiceDisplayChannelPrivate *c, guint32 surf
 static void spice_display_channel_reset(SpiceChannel *channel, gboolean migrating);
 static void spice_display_channel_reset_capabilities(SpiceChannel *channel);
 static void destroy_canvas(display_surface *surface);
+static void display_stream_destroy(gpointer st);
 static void display_session_mm_time_reset_cb(SpiceSession *session, gpointer data);
 static SpiceGlScanout* spice_gl_scanout_copy(const SpiceGlScanout *scanout);
 
@@ -129,6 +125,14 @@ spice_gl_scanout_copy(const SpiceGlScanout *scanout)
     return so;
 }
 
+/**
+ * spice_gl_scanout_free:
+ * @scanout: a #SpiceGlScanout
+ *
+ * Frees the @scanout.
+ *
+ * Since: 0.31
+ */
 void
 spice_gl_scanout_free(SpiceGlScanout *scanout)
 {
@@ -218,7 +222,7 @@ static void spice_display_get_property(GObject    *object,
         break;
     }
     case PROP_GL_SCANOUT: {
-        g_value_set_static_boxed(value, spice_display_get_gl_scanout(channel));
+        g_value_set_static_boxed(value, spice_display_channel_get_gl_scanout(channel));
         break;
     }
     default:
@@ -318,7 +322,6 @@ static void spice_display_channel_class_init(SpiceDisplayChannelClass *klass)
 
     /**
      * SpiceDisplayChannel:gl-scanout:
-     * @display: the #SpiceDisplayChannel that emitted the signal
      *
      * The last #SpiceGlScanout received.
      *
@@ -449,7 +452,27 @@ static void spice_display_channel_class_init(SpiceDisplayChannelClass *klass)
                      4,
                      G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
 
-    g_type_class_add_private(klass, sizeof(SpiceDisplayChannelPrivate));
+    /**
+     * SpiceDisplayChannel::streaming-mode:
+     * @display: the #SpiceDisplayChannel that emitted the signal
+     * @streaming_mode: %TRUE when it's streaming mode
+     *
+     * Return: handle for the display window if possible
+     *
+     * The #SpiceDisplayChannel::streaming-mode signal is emitted when
+     * spice server is working in streaming mode.
+     *
+     * Since 0.35
+     **/
+    signals[SPICE_DISPLAY_STREAMING_MODE] =
+        g_signal_new("streaming-mode",
+                     G_OBJECT_CLASS_TYPE(gobject_class),
+                     0, 0,
+                     NULL, NULL,
+                     g_cclosure_user_marshal_POINTER__BOOLEAN,
+                     G_TYPE_POINTER,
+                     1,
+                     G_TYPE_BOOLEAN);
 
     channel_set_handlers(SPICE_CHANNEL_CLASS(klass));
 }
@@ -464,9 +487,30 @@ static void spice_display_channel_class_init(SpiceDisplayChannelClass *klass)
  *
  * Returns: %TRUE if the primary surface was found and its details
  * collected in @primary.
+ *
+ * Deprecated: 0.35: use spice_display_channel_get_primary() instead.
  */
 gboolean spice_display_get_primary(SpiceChannel *channel, guint32 surface_id,
                                    SpiceDisplayPrimary *primary)
+{
+    return spice_display_channel_get_primary(channel, surface_id, primary);
+}
+
+/**
+ * spice_display_channel_get_primary:
+ * @channel: a #SpiceDisplayChannel
+ * @surface_id: a surface id
+ * @primary: a #SpiceDisplayPrimary
+ *
+ * Retrieve primary display surface @surface_id.
+ *
+ * Returns: %TRUE if the primary surface was found and its details
+ * collected in @primary.
+ *
+ * Since: 0.35
+ */
+gboolean spice_display_channel_get_primary(SpiceChannel *channel, guint32 surface_id,
+                                           SpiceDisplayPrimary *primary)
 {
     g_return_val_if_fail(SPICE_IS_DISPLAY_CHANNEL(channel), FALSE);
     g_return_val_if_fail(primary != NULL, FALSE);
@@ -500,8 +544,24 @@ gboolean spice_display_get_primary(SpiceChannel *channel, guint32 surface_id,
  * for the @channel.
  *
  * Since: 0.31
+ * Deprecated: 0.35: use spice_display_channel_change_preferred_compression() instead.
  */
 void spice_display_change_preferred_compression(SpiceChannel *channel, gint compression)
+{
+    spice_display_channel_change_preferred_compression(channel, compression);
+}
+
+/**
+ * spice_display_channel_change_preferred_compression:
+ * @channel: a #SpiceDisplayChannel
+ * @compression: a #SpiceImageCompression
+ *
+ * Tells the spice server to change the preferred image compression
+ * for the @channel.
+ *
+ * Since: 0.35
+ */
+void spice_display_channel_change_preferred_compression(SpiceChannel *channel, gint compression)
 {
     SpiceMsgOut *out;
     SpiceMsgcDisplayPreferredCompression pref_comp_msg;
@@ -523,6 +583,78 @@ void spice_display_change_preferred_compression(SpiceChannel *channel, gint comp
     spice_msg_out_send_internal(out);
 }
 
+static void spice_display_send_client_preferred_video_codecs(SpiceChannel *channel,
+                                                             const GArray *codecs)
+{
+    guint i;
+    SpiceMsgOut *out;
+    SpiceMsgcDisplayPreferredVideoCodecType *msg;
+
+    msg = g_malloc0(sizeof(SpiceMsgcDisplayPreferredVideoCodecType) +
+                    (sizeof(SpiceVideoCodecType) * codecs->len));
+    msg->num_of_codecs = codecs->len;
+    for (i = 0; i < codecs->len; i++) {
+        msg->codecs[i] = g_array_index(codecs, gint, i);
+    }
+
+    out = spice_msg_out_new(channel, SPICE_MSGC_DISPLAY_PREFERRED_VIDEO_CODEC_TYPE);
+    out->marshallers->msgc_display_preferred_video_codec_type(out->marshaller, msg);
+    spice_msg_out_send_internal(out);
+    g_free(msg);
+}
+
+/**
+ * spice_display_change_preferred_video_codec_type:
+ * @channel: a #SpiceDisplayChannel
+ * @codec_type: a #SpiceVideoCodecType
+ *
+ * Tells the spice server to change the preferred video codec type for
+ * streaming in @channel. Application can set only one preferred video codec per
+ * display channel.
+ *
+ * Since: 0.34
+ * Deprecated: 0.35: use spice_display_channel_change_preferred_video_codec_type() instead.
+ */
+void spice_display_change_preferred_video_codec_type(SpiceChannel *channel, gint codec_type)
+{
+    spice_display_channel_change_preferred_video_codec_type(channel, codec_type);
+}
+
+/**
+ * spice_display_channel_change_preferred_video_codec_type:
+ * @channel: a #SpiceDisplayChannel
+ * @codec_type: a #SpiceVideoCodecType
+ *
+ * Tells the spice server to change the preferred video codec type for
+ * streaming in @channel. Application can set only one preferred video codec per
+ * display channel.
+ *
+ * Since: 0.35
+ */
+void spice_display_channel_change_preferred_video_codec_type(SpiceChannel *channel, gint codec_type)
+{
+    GArray *codecs;
+
+    g_return_if_fail(SPICE_IS_DISPLAY_CHANNEL(channel));
+    g_return_if_fail(codec_type >= SPICE_VIDEO_CODEC_TYPE_MJPEG &&
+                     codec_type < SPICE_VIDEO_CODEC_TYPE_ENUM_END);
+
+    if (!spice_channel_test_capability(channel, SPICE_DISPLAY_CAP_PREF_VIDEO_CODEC_TYPE)) {
+        CHANNEL_DEBUG(channel, "does not have capability to change the preferred video codec type");
+        return;
+    }
+
+    /* FIXME: We should detect video codecs that client machine can do hw
+     * decoding, store this information (as GArray) and send it to the server.
+     * This array can be rearranged to have @codec_type in the front (which is
+     * the preferred for the client side) */
+    CHANNEL_DEBUG(channel, "changing preferred video codec type to %s", gst_opts[codec_type].name);
+    codecs = g_array_new(FALSE, FALSE, sizeof(gint));
+    g_array_append_val(codecs, codec_type);
+    spice_display_send_client_preferred_video_codecs(channel, codecs);
+    g_array_unref(codecs);
+}
+
 /**
  * spice_display_get_gl_scanout:
  * @channel: a #SpiceDisplayChannel
@@ -532,9 +664,26 @@ void spice_display_change_preferred_compression(SpiceChannel *channel, gint comp
  * Returns: the current GL scanout, or %NULL if none or not valid
  *
  * Since: 0.31
+ * Deprecated: 0.35: use spice_display_channel_get_gl_scanout() instead.
  **/
 const SpiceGlScanout *
 spice_display_get_gl_scanout(SpiceDisplayChannel *channel)
+{
+    return spice_display_channel_get_gl_scanout(channel);
+}
+
+/**
+ * spice_display_channel_get_gl_scanout:
+ * @channel: a #SpiceDisplayChannel
+ *
+ * Retrieves the GL scanout if available
+ *
+ * Returns: the current GL scanout, or %NULL if none or not valid
+ *
+ * Since: 0.35
+ **/
+const SpiceGlScanout *
+spice_display_channel_get_gl_scanout(SpiceDisplayChannel *channel)
 {
     g_return_val_if_fail(SPICE_IS_DISPLAY_CHANNEL(channel), NULL);
 
@@ -687,19 +836,10 @@ static SpiceImageSurfacesOps image_surfaces_ops = {
     .get = surfaces_get
 };
 
-#if defined(G_OS_WIN32)
-static HDC create_compatible_dc(void)
-{
-    HDC dc = CreateCompatibleDC(NULL);
-    if (!dc) {
-        g_warning("create compatible DC failed");
-    }
-    return dc;
-}
-#endif
-
 static void spice_display_channel_reset_capabilities(SpiceChannel *channel)
 {
+    guint i;
+
     spice_channel_set_capability(SPICE_CHANNEL(channel), SPICE_DISPLAY_CAP_SIZED_STREAM);
     spice_channel_set_capability(SPICE_CHANNEL(channel), SPICE_DISPLAY_CAP_MONITORS_CONFIG);
     spice_channel_set_capability(SPICE_CHANNEL(channel), SPICE_DISPLAY_CAP_COMPOSITE);
@@ -717,23 +857,12 @@ static void spice_display_channel_reset_capabilities(SpiceChannel *channel)
 #ifdef HAVE_BUILTIN_MJPEG
     spice_channel_set_capability(SPICE_CHANNEL(channel), SPICE_DISPLAY_CAP_CODEC_MJPEG);
 #endif
-    if (gstvideo_has_codec(SPICE_VIDEO_CODEC_TYPE_MJPEG)) {
-        spice_channel_set_capability(SPICE_CHANNEL(channel),
-                                     SPICE_DISPLAY_CAP_CODEC_MJPEG);
-    } else {
-        SPICE_DEBUG("GStreamer does not support the mjpeg codec");
-    }
-    if (gstvideo_has_codec(SPICE_VIDEO_CODEC_TYPE_VP8)) {
-        spice_channel_set_capability(SPICE_CHANNEL(channel),
-                                     SPICE_DISPLAY_CAP_CODEC_VP8);
-    } else {
-        SPICE_DEBUG("GStreamer does not support the vp8 codec");
-    }
-    if (gstvideo_has_codec(SPICE_VIDEO_CODEC_TYPE_H264)) {
-        spice_channel_set_capability(SPICE_CHANNEL(channel),
-                                     SPICE_DISPLAY_CAP_CODEC_H264);
-    } else {
-        SPICE_DEBUG("GStreamer does not support the h264 codec");
+    for (i = 1; i < G_N_ELEMENTS(gst_opts); i++) {
+        if (gstvideo_has_codec(i)) {
+            spice_channel_set_capability(SPICE_CHANNEL(channel), gst_opts[i].cap);
+        } else {
+            SPICE_DEBUG("GStreamer does not support the %s codec", gst_opts[i].name);
+        }
     }
 }
 
@@ -749,15 +878,12 @@ static void spice_display_channel_init(SpiceDisplayChannel *channel)
 {
     SpiceDisplayChannelPrivate *c;
 
-    c = channel->priv = SPICE_DISPLAY_CHANNEL_GET_PRIVATE(channel);
+    c = channel->priv = spice_display_channel_get_instance_private(channel);
 
-    c->surfaces = g_hash_table_new_full(NULL, NULL, NULL, destroy_surface);
+    c->surfaces = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, destroy_surface);
     c->image_cache.ops = &image_cache_ops;
     c->palette_cache.ops = &palette_cache_ops;
     c->image_surfaces.ops = &image_surfaces_ops;
-#if defined(G_OS_WIN32)
-    c->dc = create_compatible_dc();
-#endif
     c->monitors_max = 1;
     c->scanout.fd = -1;
 
@@ -924,7 +1050,7 @@ static void spice_display_channel_up(SpiceChannel *channel)
     g_coroutine_object_notify(G_OBJECT(channel), "monitors");
 
     if (preferred_compression != SPICE_IMAGE_COMPRESSION_INVALID) {
-        spice_display_change_preferred_compression(channel, preferred_compression);
+        spice_display_channel_change_preferred_compression(channel, preferred_compression);
     }
 }
 
@@ -1061,11 +1187,11 @@ static void display_update_stream_region(display_stream *st)
 {
     int i;
 
-    switch (st->clip->type) {
+    switch (st->clip.type) {
     case SPICE_CLIP_TYPE_RECTS:
         region_clear(&st->region);
-        for (i = 0; i < st->clip->rects->num_rects; i++) {
-            region_add(&st->region, &st->clip->rects->rects[i]);
+        for (i = 0; i < st->clip.rects->num_rects; i++) {
+            region_add(&st->region, &st->clip.rects->rects[i]);
         }
         st->have_region = true;
         break;
@@ -1076,12 +1202,94 @@ static void display_update_stream_region(display_stream *st)
     }
 }
 
+static void report_invalid_stream(SpiceChannel *channel, uint32_t id)
+{
+    if (spice_channel_test_capability(channel, SPICE_DISPLAY_CAP_STREAM_REPORT)) {
+        SpiceMsgcDisplayStreamReport report;
+        SpiceMsgOut *msg;
+
+        /* Send a special stream report (UINT_MAX dropped frames out of zero)
+         * to indicate there is no such stream.
+         */
+        spice_printerr("notify the server that stream %u does not exist", id);
+        memset(&report, 0, sizeof(report));
+        report.stream_id = id;
+        report.num_frames = 0;
+        report.num_drops = UINT_MAX;
+
+        msg = spice_msg_out_new(SPICE_CHANNEL(channel), SPICE_MSGC_DISPLAY_STREAM_REPORT);
+        msg->marshallers->msgc_display_stream_report(msg->marshaller, &report);
+        spice_msg_out_send(msg);
+    }
+}
+
+static display_stream *get_stream_by_id(SpiceChannel *channel, uint32_t id)
+{
+    SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
+
+    if (c != NULL && c->streams != NULL && id < c->nstreams &&
+        c->streams[id] != NULL) {
+        return c->streams[id];
+    }
+
+    report_invalid_stream(channel, id);
+    return NULL;
+}
+
 /* coroutine context */
+static display_stream *display_stream_create(SpiceChannel *channel,
+                                             uint32_t id, uint32_t surface_id,
+                                             uint32_t flags, uint32_t codec_type,
+                                             const SpiceRect *dest, const SpiceClip *clip)
+{
+    SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
+    display_stream *st = g_new0(display_stream, 1);
+
+    st->id = id;
+    st->flags = flags;
+    st->dest = *dest;
+    st->clip = *clip;
+    st->surface = find_surface(c, surface_id);
+    st->channel = channel;
+    st->drops_seqs_stats_arr = g_array_new(FALSE, FALSE, sizeof(drops_sequence_stats));
+
+    region_init(&st->region);
+    display_update_stream_region(st);
+
+    switch (codec_type) {
+#ifdef HAVE_BUILTIN_MJPEG
+    case SPICE_VIDEO_CODEC_TYPE_MJPEG:
+        st->video_decoder = create_mjpeg_decoder(codec_type, st);
+        break;
+#endif
+    default:
+#ifdef HAVE_GSTVIDEO
+        st->video_decoder = create_gstreamer_decoder(codec_type, st);
+#endif
+        break;
+    }
+    if (st->video_decoder == NULL) {
+        spice_printerr("could not create a video decoder for codec %u", codec_type);
+        g_clear_pointer(&st, display_stream_destroy);
+    }
+    return st;
+}
+
+static void destroy_stream(SpiceChannel *channel, int id)
+{
+    SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
+
+    g_return_if_fail(c != NULL);
+    g_return_if_fail(c->streams != NULL);
+    g_return_if_fail(c->nstreams > id);
+
+    g_clear_pointer(&c->streams[id], display_stream_destroy);
+}
+
 static void display_handle_stream_create(SpiceChannel *channel, SpiceMsgIn *in)
 {
     SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
     SpiceMsgDisplayStreamCreate *op = spice_msg_in_parsed(in);
-    display_stream *st;
 
     CHANNEL_DEBUG(channel, "%s: id %u", __FUNCTION__, op->id);
 
@@ -1097,34 +1305,14 @@ static void display_handle_stream_create(SpiceChannel *channel, SpiceMsgIn *in)
         memset(c->streams + n, 0, (c->nstreams - n) * sizeof(c->streams[0]));
     }
     g_return_if_fail(c->streams[op->id] == NULL);
-    c->streams[op->id] = g_new0(display_stream, 1);
-    st = c->streams[op->id];
 
-    st->msg_create = in;
-    spice_msg_in_ref(in);
-    st->clip = &op->clip;
-    st->surface = find_surface(c, op->surface_id);
-    st->channel = channel;
-    st->drops_seqs_stats_arr = g_array_new(FALSE, FALSE, sizeof(drops_sequence_stats));
-
-    region_init(&st->region);
-    display_update_stream_region(st);
-
-    switch (op->codec_type) {
-#ifdef HAVE_BUILTIN_MJPEG
-    case SPICE_VIDEO_CODEC_TYPE_MJPEG:
-        st->video_decoder = create_mjpeg_decoder(op->codec_type, st);
-        break;
-#endif
-    default:
-#ifdef HAVE_GSTVIDEO
-        st->video_decoder = create_gstreamer_decoder(op->codec_type, st);
-#else
-        st->video_decoder = NULL;
-#endif
-    }
-    if (st->video_decoder == NULL) {
-        spice_printerr("could not create a video decoder for codec %u", op->codec_type);
+    c->streams[op->id] = display_stream_create(channel, op->id, op->surface_id,
+                                               op->flags, op->codec_type,
+                                               &op->dest, &op->clip);
+    if (c->streams[op->id] == NULL) {
+        spice_printerr("could not create the %u video stream", op->id);
+        destroy_stream(channel, op->id);
+        report_invalid_stream(channel, op->id);
     }
 }
 
@@ -1132,9 +1320,7 @@ static const SpiceRect *stream_get_dest(display_stream *st, SpiceMsgIn *frame_ms
 {
     if (frame_msg == NULL ||
         spice_msg_in_type(frame_msg) != SPICE_MSG_DISPLAY_STREAM_DATA_SIZED) {
-        SpiceMsgDisplayStreamCreate *info = spice_msg_in_parsed(st->msg_create);
-
-        return &info->dest;
+        return &st->dest;
     } else {
         SpiceMsgDisplayStreamDataSized *op = spice_msg_in_parsed(frame_msg);
 
@@ -1143,15 +1329,7 @@ static const SpiceRect *stream_get_dest(display_stream *st, SpiceMsgIn *frame_ms
 
 }
 
-static uint32_t stream_get_flags(display_stream *st)
-{
-    SpiceMsgDisplayStreamCreate *info = spice_msg_in_parsed(st->msg_create);
-
-    return info->flags;
-}
-
-G_GNUC_INTERNAL
-uint32_t spice_msg_in_frame_data(SpiceMsgIn *frame_msg, uint8_t **data)
+static uint32_t spice_msg_in_frame_data(SpiceMsgIn *frame_msg, uint8_t **data)
 {
     switch (spice_msg_in_type(frame_msg)) {
     case SPICE_MSG_DISPLAY_STREAM_DATA: {
@@ -1186,33 +1364,37 @@ void stream_dropped_frame_on_playback(display_stream *st)
 
 /* main context */
 G_GNUC_INTERNAL
-void stream_display_frame(display_stream *st, SpiceMsgIn *frame_msg,
-                          uint32_t width, uint32_t height, uint8_t *data)
+void stream_display_frame(display_stream *st, SpiceFrame *frame,
+                          uint32_t width, uint32_t height, int stride, uint8_t *data)
 {
-    const SpiceRect *dest;
-    int stride;
-
-    dest = stream_get_dest(st, frame_msg);
-
-    stride = width * sizeof(uint32_t);
-    if (!(stream_get_flags(st) & SPICE_STREAM_FLAGS_TOP_DOWN)) {
+    if (stride == SPICE_UNKNOWN_STRIDE) {
+        stride = width * sizeof(uint32_t);
+    }
+    if (!(st->flags & SPICE_STREAM_FLAGS_TOP_DOWN)) {
         data += stride * (height - 1);
         stride = -stride;
     }
 
     st->surface->canvas->ops->put_image(st->surface->canvas,
-#ifdef G_OS_WIN32
-                                        SPICE_DISPLAY_CHANNEL(st->channel)->priv->dc,
-#endif
-                                        dest, data,
+                                        &frame->dest, data,
                                         width, height, stride,
                                         st->have_region ? &st->region : NULL);
 
-    if (st->surface->primary)
+    if (st->surface->primary) {
         g_signal_emit(st->channel, signals[SPICE_DISPLAY_INVALIDATE], 0,
-                      dest->left, dest->top,
-                      dest->right - dest->left,
-                      dest->bottom - dest->top);
+                      frame->dest.left, frame->dest.top,
+                      frame->dest.right - frame->dest.left,
+                      frame->dest.bottom - frame->dest.top);
+    }
+}
+
+guintptr get_window_handle(display_stream *st)
+{
+   void* handle = 0;
+
+   g_signal_emit(st->channel, signals[SPICE_DISPLAY_STREAMING_MODE], 0,
+                 st->surface->streaming_mode, &handle);
+   return st->surface->streaming_mode ? (guintptr)handle : 0;
 }
 
 /* after a sequence of 3 drops, push a report to the server, even
@@ -1222,9 +1404,10 @@ void stream_display_frame(display_stream *st, SpiceMsgIn *frame_msg,
 static void display_update_stream_report(SpiceDisplayChannel *channel, uint32_t stream_id,
                                          uint32_t frame_time, int32_t latency)
 {
-    display_stream *st = channel->priv->streams[stream_id];
+    display_stream *st = get_stream_by_id(SPICE_CHANNEL(channel), stream_id);
     guint64 now;
 
+    g_return_if_fail(st != NULL);
     if (!st->report_is_active) {
         return;
     }
@@ -1244,7 +1427,7 @@ static void display_update_stream_report(SpiceDisplayChannel *channel, uint32_t
     }
 
     if (st->report_num_frames >= st->report_max_window ||
-        now - st->report_start_time >= st->report_timeout ||
+        spice_mmtime_diff(now - st->report_start_time, st->report_timeout) >= 0 ||
         st->report_drops_seq_len >= STREAM_REPORT_DROP_SEQ_LEN_LIMIT) {
         SpiceMsgcDisplayStreamReport report;
         SpiceSession *session = spice_channel_get_session(SPICE_CHANNEL(channel));
@@ -1337,15 +1520,12 @@ static void display_handle_stream_data(SpiceChannel *channel, SpiceMsgIn *in)
 {
     SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
     SpiceStreamDataHeader *op = spice_msg_in_parsed(in);
-    display_stream *st;
+    display_stream *st = get_stream_by_id(channel, op->id);
     guint32 mmtime;
     int32_t latency;
+    SpiceFrame *frame;
 
-    g_return_if_fail(c != NULL);
-    g_return_if_fail(c->streams != NULL);
-    g_return_if_fail(c->nstreams > op->id);
-
-    st =  c->streams[op->id];
+    g_return_if_fail(st != NULL);
     mmtime = stream_get_time(st);
 
     if (spice_msg_in_type(in) == SPICE_MSG_DISPLAY_STREAM_DATA_SIZED) {
@@ -1390,7 +1570,20 @@ static void display_handle_stream_data(SpiceChannel *channel, SpiceMsgIn *in)
      * decoding and best decide if/when to drop them when they are late,
      * taking into account the impact on later frames.
      */
-    st->video_decoder->queue_frame(st->video_decoder, in,  latency);
+    frame = g_new(SpiceFrame, 1);
+    frame->mm_time = op->multi_media_time;
+    frame->dest = *stream_get_dest(st, in);
+    frame->size = spice_msg_in_frame_data(in, &frame->data);
+    frame->data_opaque = in;
+    frame->ref_data = (void*)spice_msg_in_ref;
+    frame->unref_data = (void*)spice_msg_in_unref;
+    frame->free = (void*)g_free;
+    if (!st->video_decoder->queue_frame(st->video_decoder, frame, latency)) {
+        destroy_stream(channel, op->id);
+        report_invalid_stream(channel, op->id);
+        return;
+    }
+
     if (c->enable_adaptive_streaming) {
         display_update_stream_report(SPICE_DISPLAY_CHANNEL(channel), op->id,
                                      op->multi_media_time, latency);
@@ -1404,66 +1597,48 @@ static void display_handle_stream_data(SpiceChannel *channel, SpiceMsgIn *in)
 /* coroutine context */
 static void display_handle_stream_clip(SpiceChannel *channel, SpiceMsgIn *in)
 {
-    SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
     SpiceMsgDisplayStreamClip *op = spice_msg_in_parsed(in);
-    display_stream *st;
+    display_stream *st = get_stream_by_id(channel, op->id);
 
-    g_return_if_fail(c != NULL);
-    g_return_if_fail(c->streams != NULL);
-    g_return_if_fail(c->nstreams > op->id);
+    g_return_if_fail(st != NULL);
 
-    st = c->streams[op->id];
-
-    if (st->msg_clip) {
-        spice_msg_in_unref(st->msg_clip);
-    }
-    spice_msg_in_ref(in);
-    st->msg_clip = in;
-    st->clip = &op->clip;
+    st->clip = op->clip;
     display_update_stream_region(st);
 }
 
-static void destroy_stream(SpiceChannel *channel, int id)
+static void display_stream_destroy(gpointer st_pointer)
 {
-    SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
-    display_stream *st;
-    guint64 drops_duration_total = 0;
-    guint32 num_out_frames;
     int i;
+    display_stream *st = st_pointer;
 
-    g_return_if_fail(c != NULL);
-    g_return_if_fail(c->streams != NULL);
-    g_return_if_fail(c->nstreams > id);
-
-    st = c->streams[id];
-    if (!st)
-        return;
-
-    num_out_frames = st->num_input_frames - st->arrive_late_count - st->num_drops_on_playback;
-    CHANNEL_DEBUG(channel, "%s: id=%d #in-frames=%u out/in=%.2f "
-        "#drops-on-receive=%u avg-late-time(ms)=%.2f "
-        "#drops-on-playback=%u", __FUNCTION__,
-        id,
-        st->num_input_frames,
-        num_out_frames / (double)st->num_input_frames,
-        st->arrive_late_count,
-        st->arrive_late_count ? st->arrive_late_time / ((double)st->arrive_late_count): 0,
-        st->num_drops_on_playback);
-    if (st->num_drops_seqs) {
-        CHANNEL_DEBUG(channel, "%s: #drops-sequences=%u ==>", __FUNCTION__, st->num_drops_seqs);
-    }
-    for (i = 0; i < st->num_drops_seqs; i++) {
+    if (st->num_input_frames > 0) {
+        guint64 drops_duration_total = 0;
+        guint32 num_out_frames = st->num_input_frames - st->arrive_late_count - st->num_drops_on_playback;
+        CHANNEL_DEBUG(st->channel, "%s: id=%u #in-frames=%u out/in=%.2f "
+            "#drops-on-receive=%u avg-late-time(ms)=%.2f "
+            "#drops-on-playback=%u", __FUNCTION__,
+            st->id,
+            st->num_input_frames,
+            num_out_frames / (double)st->num_input_frames,
+            st->arrive_late_count,
+            st->arrive_late_count ? st->arrive_late_time / ((double)st->arrive_late_count): 0,
+            st->num_drops_on_playback);
+        if (st->num_drops_seqs) {
+            CHANNEL_DEBUG(st->channel, "%s: #drops-sequences=%u ==>", __FUNCTION__, st->num_drops_seqs);
+        }
+        for (i = 0; i < st->num_drops_seqs; i++) {
             drops_sequence_stats *stats = &g_array_index(st->drops_seqs_stats_arr,
                                                          drops_sequence_stats,
                                                          i);
             drops_duration_total += stats->duration;
-            CHANNEL_DEBUG(channel, "%s: \t len=%u start-ms=%u duration-ms=%u", __FUNCTION__,
-                                   stats->len,
-                                   stats->start_mm_time - st->first_frame_mm_time,
-                                   stats->duration);
-    }
-    if (st->num_drops_seqs) {
-        CHANNEL_DEBUG(channel, "%s: drops-total-duration=%"G_GUINT64_FORMAT" ==>", __FUNCTION__, drops_duration_total);
+            CHANNEL_DEBUG(st->channel, "%s: \t len=%u start-ms=%u duration-ms=%u", __FUNCTION__,
+                          stats->len,
+                          stats->start_mm_time - st->first_frame_mm_time,
+                          stats->duration);
+        }
+        if (st->num_drops_seqs) {
+            CHANNEL_DEBUG(st->channel, "%s: drops-total-duration=%"G_GUINT64_FORMAT" ==>", __FUNCTION__, drops_duration_total);
+        }
     }
 
     g_array_free(st->drops_seqs_stats_arr, TRUE);
@@ -1472,12 +1647,7 @@ static void destroy_stream(SpiceChannel *channel, int id)
         st->video_decoder->destroy(st->video_decoder);
     }
 
-    if (st->msg_clip)
-        spice_msg_in_unref(st->msg_clip);
-    spice_msg_in_unref(st->msg_create);
-
     g_free(st);
-    c->streams[id] = NULL;
 }
 
 static void clear_streams(SpiceChannel *channel)
@@ -1511,17 +1681,10 @@ static void display_handle_stream_destroy_all(SpiceChannel *channel, SpiceMsgIn
 /* coroutine context */
 static void display_handle_stream_activate_report(SpiceChannel *channel, SpiceMsgIn *in)
 {
-    SpiceDisplayChannelPrivate *c = SPICE_DISPLAY_CHANNEL(channel)->priv;
     SpiceMsgDisplayStreamActivateReport *op = spice_msg_in_parsed(in);
-    display_stream *st;
+    display_stream *st = get_stream_by_id(channel, op->stream_id);
 
-    g_return_if_fail(c != NULL);
-    g_return_if_fail(c->streams != NULL);
-    g_return_if_fail(c->nstreams > op->stream_id);
-
-    st = c->streams[op->stream_id];
     g_return_if_fail(st != NULL);
-
     st->report_is_active = TRUE;
     st->report_id = op->unique_id;
     st->report_max_window = op->max_window_size;
@@ -1638,9 +1801,10 @@ static void display_handle_surface_create(SpiceChannel *channel, SpiceMsgIn *in)
     surface->height = create->height;
     surface->stride = create->width * 4;
     surface->size   = surface->height * surface->stride;
+    surface->streaming_mode = !!(create->flags & SPICE_SURFACE_FLAGS_STREAMING_MODE);
 
     if (create->flags & SPICE_SURFACE_FLAGS_PRIMARY) {
-        SPICE_DEBUG("primary flags: %x", create->flags);
+        SPICE_DEBUG("surface flags: %x", create->flags);
         surface->primary = true;
         create_canvas(channel, surface);
         if (c->mark_false_event_id != 0) {
@@ -1793,8 +1957,25 @@ static void display_handle_gl_draw(SpiceChannel *channel, SpiceMsgIn *in)
  * (failing to do so for each gl-draw may result in a frozen display).
  *
  * Since: 0.31
+ * Deprecated: 0.35: use spice_display_channel_gl_draw_done() instead.
  **/
 void spice_display_gl_draw_done(SpiceDisplayChannel *display)
+{
+    spice_display_channel_gl_draw_done(display);
+}
+
+/**
+ * spice_display_channel_gl_draw_done:
+ * @channel: a #SpiceDisplayChannel
+ *
+ * After a SpiceDisplayChannel::gl-draw is emitted, the client should
+ * draw the current display with the current GL scanout, and must
+ * release the GL resource with a call to spice_display_gl_draw_done()
+ * (failing to do so for each gl-draw may result in a frozen display).
+ *
+ * Since: 0.35
+ **/
+void spice_display_channel_gl_draw_done(SpiceDisplayChannel *display)
 {
     SpiceChannel *channel;
     SpiceMsgOut *out;
