@@ -40,9 +40,6 @@
  * is emitted with #SpiceInputsChannel::inputs-modifiers signal.
  */
 
-#define SPICE_INPUTS_CHANNEL_GET_PRIVATE(obj)                                  \
-    (G_TYPE_INSTANCE_GET_PRIVATE((obj), SPICE_TYPE_INPUTS_CHANNEL, SpiceInputsChannelPrivate))
-
 struct _SpiceInputsChannelPrivate {
     int                         bs;
     int                         dx, dy;
@@ -52,7 +49,7 @@ struct _SpiceInputsChannelPrivate {
     guint32                     locks;
 };
 
-G_DEFINE_TYPE(SpiceInputsChannel, spice_inputs_channel, SPICE_TYPE_CHANNEL)
+G_DEFINE_TYPE_WITH_PRIVATE(SpiceInputsChannel, spice_inputs_channel, SPICE_TYPE_CHANNEL)
 
 /* Properties */
 enum {
@@ -77,7 +74,7 @@ static void channel_set_handlers(SpiceChannelClass *klass);
 
 static void spice_inputs_channel_init(SpiceInputsChannel *channel)
 {
-    channel->priv = SPICE_INPUTS_CHANNEL_GET_PRIVATE(channel);
+    channel->priv = spice_inputs_channel_get_instance_private(channel);
 }
 
 static void spice_inputs_get_property(GObject    *object,
@@ -143,7 +140,6 @@ static void spice_inputs_channel_class_init(SpiceInputsChannelClass *klass)
                      G_TYPE_NONE,
                      0);
 
-    g_type_class_add_private(klass, sizeof(SpiceInputsChannelPrivate));
     channel_set_handlers(SPICE_CHANNEL_CLASS(klass));
 }
 
@@ -242,10 +238,8 @@ static void inputs_handle_modifiers(SpiceChannel *channel, SpiceMsgIn *in)
     SpiceInputsChannelPrivate *c = SPICE_INPUTS_CHANNEL(channel)->priv;
     SpiceMsgInputsKeyModifiers *modifiers = spice_msg_in_parsed(in);
 
-    if (c->modifiers != modifiers->modifiers) {
-        c->modifiers = modifiers->modifiers;
-        g_coroutine_signal_emit(channel, signals[SPICE_INPUTS_MODIFIERS], 0);
-    }
+    c->modifiers = modifiers->modifiers;
+    g_coroutine_signal_emit(channel, signals[SPICE_INPUTS_MODIFIERS], 0);
 }
 
 /* coroutine context */
@@ -285,10 +279,29 @@ static void channel_set_handlers(SpiceChannelClass *klass)
  * @dy: delta Y mouse coordinates
  * @button_state: SPICE_MOUSE_BUTTON_MASK flags
  *
- * Change mouse position (used in SPICE_MOUSE_MODE_CLIENT).
+ * Change mouse position (used in SPICE_MOUSE_MODE_SERVER).
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_motion() instead.
  **/
 void spice_inputs_motion(SpiceInputsChannel *channel, gint dx, gint dy,
                          gint button_state)
+{
+    spice_inputs_channel_motion(channel, dx, dy, button_state);
+}
+
+/**
+ * spice_inputs_channel_motion:
+ * @channel: a #SpiceInputsChannel
+ * @dx: delta X mouse coordinates
+ * @dy: delta Y mouse coordinates
+ * @button_state: SPICE_MOUSE_BUTTON_MASK flags
+ *
+ * Change mouse position (used in SPICE_MOUSE_MODE_SERVER).
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_motion(SpiceInputsChannel *channel, gint dx, gint dy,
+                                 gint button_state)
 {
     SpiceInputsChannelPrivate *c;
 
@@ -319,9 +332,29 @@ void spice_inputs_motion(SpiceInputsChannel *channel, gint dx, gint dy,
  * @button_state: SPICE_MOUSE_BUTTON_MASK flags
  *
  * Change mouse position (used in SPICE_MOUSE_MODE_CLIENT).
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_position() instead.
  **/
 void spice_inputs_position(SpiceInputsChannel *channel, gint x, gint y,
                            gint display, gint button_state)
+{
+    spice_inputs_channel_position(channel, x, y, display, button_state);
+}
+
+/**
+ * spice_inputs_channel_position:
+ * @channel: a #SpiceInputsChannel
+ * @x: X mouse coordinates
+ * @y: Y mouse coordinates
+ * @display: display channel id
+ * @button_state: SPICE_MOUSE_BUTTON_MASK flags
+ *
+ * Change mouse position (used in SPICE_MOUSE_MODE_CLIENT).
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_position(SpiceInputsChannel *channel, gint x, gint y,
+                                   gint display, gint button_state)
 {
     SpiceInputsChannelPrivate *c;
 
@@ -350,9 +383,27 @@ void spice_inputs_position(SpiceInputsChannel *channel, gint x, gint y,
  * @button_state: SPICE_MOUSE_BUTTON_MASK flags
  *
  * Press a mouse button.
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_button_press() instead.
  **/
 void spice_inputs_button_press(SpiceInputsChannel *channel, gint button,
                                gint button_state)
+{
+    spice_inputs_channel_button_press(channel, button, button_state);
+}
+
+/**
+ * spice_inputs_channel_button_press:
+ * @channel: a #SpiceInputsChannel
+ * @button: a SPICE_MOUSE_BUTTON
+ * @button_state: SPICE_MOUSE_BUTTON_MASK flags
+ *
+ * Press a mouse button.
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_button_press(SpiceInputsChannel *channel, gint button,
+                                       gint button_state)
 {
     SpiceInputsChannelPrivate *c;
     SpiceMsgcMousePress press;
@@ -397,9 +448,27 @@ void spice_inputs_button_press(SpiceInputsChannel *channel, gint button,
  * @button_state: SPICE_MOUSE_BUTTON_MASK flags
  *
  * Release a button.
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_button_release() instead.
  **/
 void spice_inputs_button_release(SpiceInputsChannel *channel, gint button,
                                  gint button_state)
+{
+    spice_inputs_channel_button_release(channel, button, button_state);
+}
+
+/**
+ * spice_inputs_channel_button_release:
+ * @channel: a #SpiceInputsChannel
+ * @button: a SPICE_MOUSE_BUTTON
+ * @button_state: SPICE_MOUSE_BUTTON_MASK flags
+ *
+ * Release a button.
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_button_release(SpiceInputsChannel *channel, gint button,
+                                         gint button_state)
 {
     SpiceInputsChannelPrivate *c;
     SpiceMsgcMouseRelease release;
@@ -444,8 +513,25 @@ void spice_inputs_button_release(SpiceInputsChannel *channel, gint button,
  *            prefix, drop the prefix and OR the scancode with %0x100.
  *
  * Press a key.
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_key_press() instead.
  **/
 void spice_inputs_key_press(SpiceInputsChannel *channel, guint scancode)
+{
+    spice_inputs_channel_key_press(channel, scancode);
+}
+
+/**
+ * spice_inputs_channel_key_press:
+ * @channel: a #SpiceInputsChannel
+ * @scancode: a PC XT (set 1) key scancode.  For scancodes with an %0xe0
+ *            prefix, drop the prefix and OR the scancode with %0x100.
+ *
+ * Press a key.
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_key_press(SpiceInputsChannel *channel, guint scancode)
 {
     SpiceMsgcKeyDown down;
     SpiceMsgOut *msg;
@@ -470,8 +556,25 @@ void spice_inputs_key_press(SpiceInputsChannel *channel, guint scancode)
  *            prefix, drop the prefix and OR the scancode with %0x100.
  *
  * Release a key.
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_key_release() instead.
  **/
 void spice_inputs_key_release(SpiceInputsChannel *channel, guint scancode)
+{
+    spice_inputs_channel_key_release(channel, scancode);
+}
+
+/**
+ * spice_inputs_channel_key_release:
+ * @channel: a #SpiceInputsChannel
+ * @scancode: a PC XT (set 1) key scancode.  For scancodes with an %0xe0
+ *            prefix, drop the prefix and OR the scancode with %0x100.
+ *
+ * Release a key.
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_key_release(SpiceInputsChannel *channel, guint scancode)
 {
     SpiceMsgcKeyUp up;
     SpiceMsgOut *msg;
@@ -498,8 +601,25 @@ void spice_inputs_key_release(SpiceInputsChannel *channel, guint scancode)
  * Press and release a key event atomically (in the same message).
  *
  * Since: 0.13
+ *
+ * Deprecated: 0.35
  **/
 void spice_inputs_key_press_and_release(SpiceInputsChannel *input_channel, guint scancode)
+{
+    spice_inputs_channel_key_press_and_release(input_channel, scancode);
+}
+
+/**
+ * spice_inputs_channel_key_press_and_release:
+ * @channel: a #SpiceInputsChannel
+ * @scancode: a PC XT (set 1) key scancode.  For scancodes with an %0xe0
+ *            prefix, drop the prefix and OR the scancode with %0x100.
+ *
+ * Press and release a key event atomically (in the same message).
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_key_press_and_release(SpiceInputsChannel *input_channel, guint scancode)
 {
     SpiceChannel *channel = SPICE_CHANNEL(input_channel);
 
@@ -533,8 +653,8 @@ void spice_inputs_key_press_and_release(SpiceInputsChannel *input_channel, guint
         spice_msg_out_send(msg);
     } else {
         CHANNEL_DEBUG(channel, "The server doesn't support atomic press and release");
-        spice_inputs_key_press(input_channel, scancode);
-        spice_inputs_key_release(input_channel, scancode);
+        spice_inputs_channel_key_press(input_channel, scancode);
+        spice_inputs_channel_key_release(input_channel, scancode);
     }
 }
 
@@ -568,8 +688,24 @@ static SpiceMsgOut* set_key_locks(SpiceInputsChannel *channel, guint locks)
  * @locks: #SpiceInputsLock modifiers flags
  *
  * Set the keyboard locks on the guest (Caps, Num, Scroll..)
+ *
+ * Deprecated: 0.35: use spice_inputs_channel_set_key_locks() instead.
  **/
 void spice_inputs_set_key_locks(SpiceInputsChannel *channel, guint locks)
+{
+    spice_inputs_channel_set_key_locks(channel, locks);
+}
+
+/**
+ * spice_inputs_channel_set_key_locks:
+ * @channel: a #SpiceInputsChannel
+ * @locks: #SpiceInputsLock modifiers flags
+ *
+ * Set the keyboard locks on the guest (Caps, Num, Scroll..)
+ *
+ * Since: 0.35
+ **/
+void spice_inputs_channel_set_key_locks(SpiceInputsChannel *channel, guint locks)
 {
     SpiceMsgOut *msg;
 
