@@ -25,7 +25,7 @@
 #include "spice-marshal.h"
 #include "bio-gio.h"
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <openssl/rsa.h>
 #include <openssl/evp.h>
@@ -55,6 +55,15 @@ static void spice_channel_reset_capabilities(SpiceChannel *channel);
 static void spice_channel_send_migration_handshake(SpiceChannel *channel);
 static gboolean channel_connect(SpiceChannel *channel, gboolean tls);
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000 || defined(LIBRESSL_VERSION_NUMBER)
+static RSA *EVP_PKEY_get0_RSA(EVP_PKEY *pkey)
+{
+    if (pkey->type != EVP_PKEY_RSA) {
+        return NULL;
+    }
+    return pkey->pkey.rsa;
+}
+#endif
 /**
  * SECTION:spice-channel
  * @short_description: the base channel class
@@ -69,13 +78,8 @@ static gboolean channel_connect(SpiceChannel *channel, gboolean tls);
  * #SpiceInputsChannel.
  */
 
-/* ------------------------------------------------------------------ */
-/* gobject glue                                                       */
-
-#define SPICE_CHANNEL_GET_PRIVATE(obj)                                  \
-    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), SPICE_TYPE_CHANNEL, SpiceChannelPrivate))
-
 G_DEFINE_TYPE_WITH_CODE (SpiceChannel, spice_channel, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (SpiceChannel)
                          g_type_add_class_private (g_define_type_id, sizeof (SpiceChannelClassPrivate)));
 
 /* Properties */
@@ -85,6 +89,7 @@ enum {
     PROP_CHANNEL_TYPE,
     PROP_CHANNEL_ID,
     PROP_TOTAL_READ_BYTES,
+    PROP_SOCKET,
 };
 
 /* Signals */
@@ -104,7 +109,7 @@ static void spice_channel_init(SpiceChannel *channel)
 {
     SpiceChannelPrivate *c;
 
-    c = channel->priv = SPICE_CHANNEL_GET_PRIVATE(channel);
+    c = channel->priv = spice_channel_get_instance_private(channel);
 
     c->out_serial = 1;
     c->in_serial = 1;
@@ -215,6 +220,9 @@ static void spice_channel_get_property(GObject    *gobject,
     case PROP_TOTAL_READ_BYTES:
         g_value_set_ulong(value, c->total_read_bytes);
         break;
+    case PROP_SOCKET:
+        g_value_set_object(value, c->sock);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
         break;
@@ -317,6 +325,25 @@ static void spice_channel_class_init(SpiceChannelClass *klass)
                             G_PARAM_READABLE |
                             G_PARAM_STATIC_STRINGS));
 
+    /**
+     * SpiceChannel:socket:
+     *
+     * Get the underlying #GSocket. Note that you should not read or
+     * write any data to it directly since this will likely corrupt
+     * the channel stream.  This property is mainly useful to get some
+     * connections details.
+     *
+     * Since: 0.33
+     */
+    g_object_class_install_property
+        (gobject_class, PROP_SOCKET,
+         g_param_spec_object("socket",
+                             "Socket",
+                             "Underlying GSocket",
+                             G_TYPE_SOCKET,
+                             G_PARAM_READABLE |
+                             G_PARAM_STATIC_STRINGS));
+
     /**
      * SpiceChannel::channel-event:
      * @channel: the channel that emitted the signal
@@ -358,8 +385,6 @@ static void spice_channel_class_init(SpiceChannelClass *klass)
                      1,
                      G_TYPE_INT);
 
-    g_type_class_add_private(klass, sizeof(SpiceChannelPrivate));
-
     SSL_library_init();
     SSL_load_error_strings();
 }
@@ -736,6 +761,53 @@ void spice_msg_out_send_internal(SpiceMsgOut *out)
     spice_channel_write_msg(out->channel, out);
 }
 
+/*
+ * Helper function to deal with the nonblocking part of _flush_wire() function.
+ * It returns the result of the write and will set the proper bits in @cond in
+ * case the write function would block.
+ *
+ * Returns -1 in case of any problems.
+ */
+/* coroutine context */
+static gint spice_channel_flush_wire_nonblocking(SpiceChannel *channel,
+                                                 const gchar *ptr,
+                                                 size_t len,
+                                                 GIOCondition *cond)
+{
+    SpiceChannelPrivate *c = channel->priv;
+    gssize ret;
+
+    g_assert(cond != NULL);
+    *cond = 0;
+
+    if (c->tls) {
+        ret = SSL_write(c->ssl, ptr, len);
+        if (ret < 0) {
+            ret = SSL_get_error(c->ssl, ret);
+            if (ret == SSL_ERROR_WANT_READ)
+                *cond |= G_IO_IN;
+            if (ret == SSL_ERROR_WANT_WRITE)
+                *cond |= G_IO_OUT;
+            ret = -1;
+        }
+    } else {
+        GError *error = NULL;
+        ret = g_pollable_output_stream_write_nonblocking(G_POLLABLE_OUTPUT_STREAM(c->out),
+                                                         ptr, len, NULL, &error);
+        if (ret < 0) {
+            if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
+                *cond = G_IO_OUT;
+            } else {
+                CHANNEL_DEBUG(channel, "Send error %s", error->message);
+            }
+            g_clear_error(&error);
+            ret = -1;
+        }
+    }
+
+    return ret;
+}
+
 /*
  * Write all 'data' of length 'datalen' bytes out to
  * the wire
@@ -752,34 +824,10 @@ static void spice_channel_flush_wire(SpiceChannel *channel,
 
     while (offset < datalen) {
         gssize ret;
-        GError *error = NULL;
 
         if (c->has_error) return;
 
-        cond = 0;
-        if (c->tls) {
-            ret = SSL_write(c->ssl, ptr+offset, datalen-offset);
-            if (ret < 0) {
-                ret = SSL_get_error(c->ssl, ret);
-                if (ret == SSL_ERROR_WANT_READ)
-                    cond |= G_IO_IN;
-                if (ret == SSL_ERROR_WANT_WRITE)
-                    cond |= G_IO_OUT;
-                ret = -1;
-            }
-        } else {
-            ret = g_pollable_output_stream_write_nonblocking(G_POLLABLE_OUTPUT_STREAM(c->out),
-                                                             ptr+offset, datalen-offset, NULL, &error);
-            if (ret < 0) {
-                if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
-                    cond = G_IO_OUT;
-                } else {
-                    CHANNEL_DEBUG(channel, "Send error %s", error->message);
-                }
-                g_clear_error(&error);
-                ret = -1;
-            }
-        }
+        ret = spice_channel_flush_wire_nonblocking(channel, ptr+offset, datalen-offset, &cond);
         if (ret == -1) {
             if (cond != 0) {
                 // TODO: should use g_pollable_input/output_stream_create_source() in 2.28 ?
@@ -829,13 +877,12 @@ static void spice_channel_flush_sasl(SpiceChannel *channel, const void *data, si
 static void spice_channel_write(SpiceChannel *channel, const void *data, size_t len)
 {
 #ifdef HAVE_SASL
-    SpiceChannelPrivate *c = channel->priv;
-
-    if (c->sasl_conn)
+    if (channel->priv->sasl_conn) {
         spice_channel_flush_sasl(channel, data, len);
-    else
+        return;
+    }
 #endif
-        spice_channel_flush_wire(channel, data, len);
+    spice_channel_flush_wire(channel, data, len);
 }
 
 /* coroutine context */
@@ -939,29 +986,32 @@ gint spice_channel_unix_read_fd(SpiceChannel *channel)
 #endif
 
 /*
- * Read at least 1 more byte of data straight off the wire
- * into the requested buffer.
+ * Helper function to deal with the nonblocking part of _read_wire() function.
+ * It returns the result of the read and will set the proper bits in @cond in
+ * case the read function would block.
+ *
+ * Returns -1 in case of any problems.
  */
 /* coroutine context */
-static int spice_channel_read_wire(SpiceChannel *channel, void *data, size_t len)
+static int spice_channel_read_wire_nonblocking(SpiceChannel *channel,
+                                               void *data,
+                                               size_t len,
+                                               GIOCondition *cond)
 {
     SpiceChannelPrivate *c = channel->priv;
     gssize ret;
-    GIOCondition cond;
 
-reread:
+    g_assert(cond != NULL);
+    *cond = 0;
 
-    if (c->has_error) return 0; /* has_error is set by disconnect(), return no error */
-
-    cond = 0;
     if (c->tls) {
         ret = SSL_read(c->ssl, data, len);
         if (ret < 0) {
             ret = SSL_get_error(c->ssl, ret);
             if (ret == SSL_ERROR_WANT_READ)
-                cond |= G_IO_IN;
+                *cond |= G_IO_IN;
             if (ret == SSL_ERROR_WANT_WRITE)
-                cond |= G_IO_OUT;
+                *cond |= G_IO_OUT;
             ret = -1;
         }
     } else {
@@ -970,7 +1020,7 @@ reread:
                                                        data, len, NULL, &error);
         if (ret < 0) {
             if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
-                cond = G_IO_IN;
+                *cond = G_IO_IN;
             } else {
                 CHANNEL_DEBUG(channel, "Read error %s", error->message);
             }
@@ -979,25 +1029,49 @@ reread:
         }
     }
 
-    if (ret == -1) {
-        if (cond != 0) {
-            // TODO: should use g_pollable_input/output_stream_create_source() ?
-            g_coroutine_socket_wait(&c->coroutine, c->sock, cond);
-            goto reread;
-        } else {
-            c->has_error = TRUE;
-            return -errno;
-        }
-    }
-    if (ret == 0) {
-        CHANNEL_DEBUG(channel, "Closing the connection: spice_channel_read() - ret=0");
-        c->has_error = TRUE;
-        return 0;
-    }
-
     return ret;
 }
 
+/*
+ * Read at least 1 more byte of data straight off the wire
+ * into the requested buffer.
+ */
+/* coroutine context */
+static int spice_channel_read_wire(SpiceChannel *channel, void *data, size_t len)
+{
+    SpiceChannelPrivate *c = channel->priv;
+
+    while (TRUE) {
+        gssize ret;
+        GIOCondition cond;
+
+        if (c->has_error) {
+            /* has_error is set by disconnect(), return no error */
+            return 0;
+        }
+
+        ret = spice_channel_read_wire_nonblocking(channel, data, len, &cond);
+
+        if (ret == -1) {
+            if (cond != 0) {
+                // TODO: should use g_pollable_input/output_stream_create_source() ?
+                g_coroutine_socket_wait(&c->coroutine, c->sock, cond);
+                continue;
+            } else {
+                c->has_error = TRUE;
+                return -errno;
+            }
+        }
+        if (ret == 0) {
+            CHANNEL_DEBUG(channel, "Closing the connection: spice_channel_read() - ret=0");
+            c->has_error = TRUE;
+            return 0;
+        }
+
+        return ret;
+    }
+}
+
 #ifdef HAVE_SASL
 /*
  * Read at least 1 more byte of data out of the SASL decrypted
@@ -1081,28 +1155,38 @@ static int spice_channel_read(SpiceChannel *channel, void *data, size_t length)
     return length;
 }
 
+#if HAVE_SASL
 /* coroutine context */
-static void spice_channel_failed_authentication(SpiceChannel *channel,
-                                                gboolean invalidPassword)
+static void spice_channel_failed_sasl_authentication(SpiceChannel *channel)
+{
+    SpiceChannelPrivate *c = channel->priv;
+    gint err_code; /* The application should activate the authentication window fields accordingly */
+
+    if (c->auth_needs_username && c->auth_needs_password)
+        err_code = SPICE_CLIENT_ERROR_AUTH_NEEDS_PASSWORD_AND_USERNAME;
+    else if (c->auth_needs_username)
+        err_code = SPICE_CLIENT_ERROR_AUTH_NEEDS_USERNAME;
+    else
+        err_code = SPICE_CLIENT_ERROR_AUTH_NEEDS_PASSWORD;
+
+    g_set_error_literal(&c->error,
+                        SPICE_CLIENT_ERROR,
+                        err_code,
+                        _("Authentication failed"));
+
+    c->event = SPICE_CHANNEL_ERROR_AUTH;
+
+    c->has_error = TRUE; /* force disconnect */
+}
+#endif
+
+/* coroutine context */
+static void spice_channel_failed_spice_authentication(SpiceChannel *channel,
+                                                      gboolean invalidPassword)
 {
     SpiceChannelPrivate *c = channel->priv;
 
-    if (c->auth_needs_username && c->auth_needs_password)
-        g_set_error_literal(&c->error,
-                            SPICE_CLIENT_ERROR,
-                            SPICE_CLIENT_ERROR_AUTH_NEEDS_PASSWORD_AND_USERNAME,
-                            _("Authentication failed: password and username are required"));
-    else if (c->auth_needs_username)
-        g_set_error_literal(&c->error,
-                            SPICE_CLIENT_ERROR,
-                            SPICE_CLIENT_ERROR_AUTH_NEEDS_USERNAME,
-                            _("Authentication failed: username is required"));
-    else if (c->auth_needs_password)
-        g_set_error_literal(&c->error,
-                            SPICE_CLIENT_ERROR,
-                            SPICE_CLIENT_ERROR_AUTH_NEEDS_PASSWORD,
-                            _("Authentication failed: password is required"));
-    else if (invalidPassword)
+    if (invalidPassword)
         g_set_error_literal(&c->error,
                             SPICE_CLIENT_ERROR,
                             SPICE_CLIENT_ERROR_AUTH_NEEDS_PASSWORD,
@@ -1111,7 +1195,7 @@ static void spice_channel_failed_authentication(SpiceChannel *channel,
         g_set_error_literal(&c->error,
                             SPICE_CLIENT_ERROR,
                             SPICE_CLIENT_ERROR_AUTH_NEEDS_PASSWORD,
-                            _("Authentication failed: password is required"));
+                            _("Authentication failed: wrong password?"));
 
     c->event = SPICE_CHANNEL_ERROR_AUTH;
 
@@ -1138,7 +1222,7 @@ static SpiceChannelEvent spice_channel_send_spice_ticket(SpiceChannel *channel)
     pubkey = d2i_PUBKEY_bio(bioKey, NULL);
     g_return_val_if_fail(pubkey != NULL, ret);
 
-    rsa = pubkey->pkey.rsa;
+    rsa = EVP_PKEY_get0_RSA(pubkey);
     nRSASize = RSA_size(rsa);
 
     encrypted = g_alloca(nRSASize);
@@ -1150,7 +1234,7 @@ static SpiceChannelEvent spice_channel_send_spice_ticket(SpiceChannel *channel)
     if (password == NULL)
         password = g_strdup("");
     if (strlen(password) > SPICE_MAX_PASSWORD_LENGTH) {
-        spice_channel_failed_authentication(channel, TRUE);
+        spice_channel_failed_spice_authentication(channel, TRUE);
         ret = SPICE_CHANNEL_ERROR_AUTH;
         goto cleanup;
     }
@@ -1186,7 +1270,7 @@ static gboolean spice_channel_recv_auth(SpiceChannel *channel)
 
     if (link_res != SPICE_LINK_ERR_OK) {
         CHANNEL_DEBUG(channel, "link result: reply %u", link_res);
-        spice_channel_failed_authentication(channel, FALSE);
+        spice_channel_failed_spice_authentication(channel, FALSE);
         return FALSE;
     }
 
@@ -1220,10 +1304,12 @@ static void spice_channel_send_link(SpiceChannel *channel)
 {
     SpiceChannelPrivate *c = channel->priv;
     uint8_t *buffer, *p;
+    uint32_t *caps;
     int protocol, i;
+    SpiceLinkMess link_msg;
 
     c->link_hdr.magic = SPICE_MAGIC;
-    c->link_hdr.size = sizeof(c->link_msg);
+    c->link_hdr.size = sizeof(link_msg);
 
     g_object_get(c->session, "protocol", &protocol, NULL);
     switch (protocol) {
@@ -1247,13 +1333,13 @@ static void spice_channel_send_link(SpiceChannel *channel)
     c->link_hdr.major_version = GUINT32_TO_LE(c->link_hdr.major_version);
     c->link_hdr.minor_version = GUINT32_TO_LE(c->link_hdr.minor_version);
 
-    c->link_msg.connection_id = GUINT32_TO_LE(spice_session_get_connection_id(c->session));
-    c->link_msg.channel_type  = c->channel_type;
-    c->link_msg.channel_id    = c->channel_id;
-    c->link_msg.caps_offset   = GUINT32_TO_LE(sizeof(c->link_msg));
+    link_msg.connection_id = GUINT32_TO_LE(spice_session_get_connection_id(c->session));
+    link_msg.channel_type  = c->channel_type;
+    link_msg.channel_id    = c->channel_id;
+    link_msg.caps_offset   = GUINT32_TO_LE(sizeof(link_msg));
 
-    c->link_msg.num_common_caps = GUINT32_TO_LE(c->common_caps->len);
-    c->link_msg.num_channel_caps = GUINT32_TO_LE(c->caps->len);
+    link_msg.num_common_caps = GUINT32_TO_LE(c->common_caps->len);
+    link_msg.num_channel_caps = GUINT32_TO_LE(c->caps->len);
     c->link_hdr.size += (c->common_caps->len + c->caps->len) * sizeof(uint32_t);
 
     buffer = g_malloc0(sizeof(c->link_hdr) + c->link_hdr.size);
@@ -1261,17 +1347,20 @@ static void spice_channel_send_link(SpiceChannel *channel)
 
     c->link_hdr.size = GUINT32_TO_LE(c->link_hdr.size);
 
-    memcpy(p, &c->link_hdr, sizeof(c->link_hdr)); p += sizeof(c->link_hdr);
-    memcpy(p, &c->link_msg, sizeof(c->link_msg)); p += sizeof(c->link_msg);
+    memcpy(p, &c->link_hdr, sizeof(c->link_hdr));
+    p += sizeof(c->link_hdr);
 
+    memcpy(p, &link_msg, sizeof(link_msg));
+    p += sizeof(link_msg);
+
+    caps = SPICE_UNALIGNED_CAST(uint32_t *,p);
     for (i = 0; i < c->common_caps->len; i++) {
-        *(uint32_t *)p = GUINT32_TO_LE(g_array_index(c->common_caps, uint32_t, i));
-        p += sizeof(uint32_t);
+        *caps++ = GUINT32_TO_LE(g_array_index(c->common_caps, uint32_t, i));
     }
     for (i = 0; i < c->caps->len; i++) {
-        *(uint32_t *)p = GUINT32_TO_LE(g_array_index(c->caps, uint32_t, i));
-        p += sizeof(uint32_t);
+        *caps++ = GUINT32_TO_LE(g_array_index(c->caps, uint32_t, i));
     }
+    p = (uint8_t *) caps;
     CHANNEL_DEBUG(channel, "channel type %d id %d num common caps %u num caps %u",
                   c->channel_type,
                   c->channel_id,
@@ -1413,6 +1502,12 @@ spice_channel_gather_sasl_credentials(SpiceChannel *channel,
     return ret;
 }
 
+static void spice_channel_write_u32_le(SpiceChannel *channel, guint32 n)
+{
+    n = GUINT32_TO_LE(n);
+    spice_channel_write(channel, &n, sizeof(n));
+}
+
 /*
  *
  * Init msg from server
@@ -1612,17 +1707,17 @@ restart:
 
     /* Send back the chosen mechname */
     len = strlen(mechname);
-    spice_channel_write(channel, &len, sizeof(guint32));
+    spice_channel_write_u32_le(channel, len);
     spice_channel_write(channel, mechname, len);
 
     /* NB, distinction of NULL vs "" is *critical* in SASL */
     if (clientout) {
         len = clientoutlen + 1;
-        spice_channel_write(channel, &len, sizeof(guint32));
+        spice_channel_write_u32_le(channel, len);
         spice_channel_write(channel, clientout, len);
     } else {
         len = 0;
-        spice_channel_write(channel, &len, sizeof(guint32));
+        spice_channel_write_u32_le(channel, len);
     }
 
     if (c->has_error)
@@ -1698,11 +1793,11 @@ restart:
         /* NB, distinction of NULL vs "" is *critical* in SASL */
         if (clientout) {
             len = clientoutlen + 1;
-            spice_channel_write(channel, &len, sizeof(guint32));
+            spice_channel_write_u32_le(channel, len);
             spice_channel_write(channel, clientout, len);
         } else {
             len = 0;
-            spice_channel_write(channel, &len, sizeof(guint32));
+            spice_channel_write_u32_le(channel, len);
         }
 
         if (c->has_error)
@@ -1776,7 +1871,7 @@ error:
     if (saslconn)
         sasl_dispose(&saslconn);
 
-    spice_channel_failed_authentication(channel, FALSE);
+    spice_channel_failed_sasl_authentication(channel);
     ret = FALSE;
 
 cleanup:
@@ -1789,12 +1884,33 @@ cleanup:
 }
 #endif /* HAVE_SASL */
 
+/* coroutine context */
+static void store_caps(const uint8_t *caps_src, uint32_t ncaps,
+                       GArray *caps_dst)
+{
+    uint32_t *caps;
+    guint i;
+
+    g_array_set_size(caps_dst, ncaps);
+    if (ncaps == 0)
+        return;
+
+    caps = &g_array_index(caps_dst, uint32_t, 0);
+    memcpy(caps, caps_src, ncaps * sizeof(uint32_t));
+
+    for (i = 0; i < ncaps; i++, caps++) {
+        *caps = GUINT32_FROM_LE(*caps);
+        SPICE_DEBUG("\t%u:0x%X", i, *caps);
+    }
+}
+
 /* coroutine context */
 static gboolean spice_channel_recv_link_msg(SpiceChannel *channel)
 {
     SpiceChannelPrivate *c;
-    int rc, num_caps, i;
-    uint32_t *caps, num_channel_caps, num_common_caps;
+    int rc, num_caps;
+    uint32_t num_channel_caps, num_common_caps;
+    uint8_t *caps_src;
     SpiceChannelEvent event = SPICE_CHANNEL_ERROR_LINK;
 
     g_return_val_if_fail(channel != NULL, FALSE);
@@ -1834,19 +1950,13 @@ static gboolean spice_channel_recv_link_msg(SpiceChannel *channel)
     /* see original spice/client code: */
     /* g_return_if_fail(c->peer_msg + c->peer_msg->caps_offset * sizeof(uint32_t) > c->peer_msg + c->peer_hdr.size); */
 
-    caps = (uint32_t *)((uint8_t *)c->peer_msg + GUINT32_FROM_LE(c->peer_msg->caps_offset));
+    caps_src = (uint8_t *)c->peer_msg + GUINT32_FROM_LE(c->peer_msg->caps_offset);
+    CHANNEL_DEBUG(channel, "got remote common caps:");
+    store_caps(caps_src, num_common_caps, c->remote_common_caps);
 
-    g_array_set_size(c->remote_common_caps, num_common_caps);
-    for (i = 0; i < num_common_caps; i++, caps++) {
-        g_array_index(c->remote_common_caps, uint32_t, i) = GUINT32_FROM_LE(*caps);
-        CHANNEL_DEBUG(channel, "got common caps %d:0x%X", i, GUINT32_FROM_LE(*caps));
-    }
-
-    g_array_set_size(c->remote_caps, num_channel_caps);
-    for (i = 0; i < num_channel_caps; i++, caps++) {
-        g_array_index(c->remote_caps, uint32_t, i) = GUINT32_FROM_LE(*caps);
-        CHANNEL_DEBUG(channel, "got channel caps %d:0x%X", i, GUINT32_FROM_LE(*caps));
-    }
+    caps_src += num_common_caps * sizeof(uint32_t);
+    CHANNEL_DEBUG(channel, "got remote channel caps:");
+    store_caps(caps_src, num_channel_caps, c->remote_caps);
 
     if (!spice_channel_test_common_capability(channel,
             SPICE_COMMON_CAP_PROTOCOL_AUTH_SELECTION)) {
@@ -1859,14 +1969,14 @@ static gboolean spice_channel_recv_link_msg(SpiceChannel *channel)
 #ifdef HAVE_SASL
         if (spice_channel_test_common_capability(channel, SPICE_COMMON_CAP_AUTH_SASL)) {
             CHANNEL_DEBUG(channel, "Choosing SASL mechanism");
-            auth.auth_mechanism = SPICE_COMMON_CAP_AUTH_SASL;
+            auth.auth_mechanism = GUINT32_TO_LE(SPICE_COMMON_CAP_AUTH_SASL);
             spice_channel_write(channel, &auth, sizeof(auth));
             if (!spice_channel_perform_auth_sasl(channel))
                 return FALSE;
         } else
 #endif
         if (spice_channel_test_common_capability(channel, SPICE_COMMON_CAP_AUTH_SPICE)) {
-            auth.auth_mechanism = SPICE_COMMON_CAP_AUTH_SPICE;
+            auth.auth_mechanism = GUINT32_TO_LE(SPICE_COMMON_CAP_AUTH_SPICE);
             spice_channel_write(channel, &auth, sizeof(auth));
             if ((event = spice_channel_send_spice_ticket(channel)) != SPICE_CHANNEL_NONE)
                 goto error;
@@ -2031,7 +2141,7 @@ const gchar* spice_channel_type_to_string(gint type)
         str = to_string[type];
     }
 
-    return str ? str : "unknown channel type";
+    return str ? str : "unknown";
 }
 
 /**
@@ -2329,18 +2439,9 @@ static gboolean spice_channel_delayed_unref(gpointer data)
     return FALSE;
 }
 
-static X509_LOOKUP_METHOD spice_x509_mem_lookup = {
-    "spice_x509_mem_lookup",
-    0
-};
-
 static int spice_channel_load_ca(SpiceChannel *channel)
 {
     SpiceChannelPrivate *c = channel->priv;
-    STACK_OF(X509_INFO) *inf;
-    X509_INFO *itmp;
-    X509_LOOKUP *lookup;
-    BIO *in;
     int i, count = 0;
     guint8 *ca;
     guint size;
@@ -2349,25 +2450,30 @@ static int spice_channel_load_ca(SpiceChannel *channel)
 
     g_return_val_if_fail(c->ctx != NULL, 0);
 
-    lookup = X509_STORE_add_lookup(c->ctx->cert_store, &spice_x509_mem_lookup);
     ca_file = spice_session_get_ca_file(c->session);
     spice_session_get_ca(c->session, &ca, &size);
 
     CHANNEL_DEBUG(channel, "Load CA, file: %s, data: %p", ca_file, ca);
 
     if (ca != NULL) {
+        STACK_OF(X509_INFO) *inf;
+        X509_STORE *store;
+        BIO *in;
+
+        store = SSL_CTX_get_cert_store(c->ctx);
         in = BIO_new_mem_buf(ca, size);
         inf = PEM_X509_INFO_read_bio(in, NULL, NULL, NULL);
         BIO_free(in);
 
         for (i = 0; i < sk_X509_INFO_num(inf); i++) {
+            X509_INFO *itmp;
             itmp = sk_X509_INFO_value(inf, i);
             if (itmp->x509) {
-                X509_STORE_add_cert(lookup->store_ctx, itmp->x509);
+                X509_STORE_add_cert(store, itmp->x509);
                 count++;
             }
             if (itmp->crl) {
-                X509_STORE_add_crl(lookup->store_ctx, itmp->crl);
+                X509_STORE_add_crl(store, itmp->crl);
                 count++;
             }
         }
@@ -2423,7 +2529,7 @@ static void *spice_channel_coroutine(void *data)
     int rc, delay_val = 1;
     /* When some other SSL/TLS version becomes obsolete, add it to this
      * variable. */
-    long ssl_options = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
+    long ssl_options = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1;
 
     CHANNEL_DEBUG(channel, "Started background coroutine %p", &c->coroutine);
 
