@@ -18,6 +18,7 @@
 #include "config.h"
 
 #include <math.h>
+#include <gdk/gdk.h>
 
 #define EGL_EGLEXT_PROTOTYPES
 #define GL_GLEXT_PROTOTYPES
@@ -27,7 +28,9 @@
 #include "spice-gtk-session-priv.h"
 #include <libdrm/drm_fourcc.h>
 
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#endif
 #ifdef GDK_WINDOWING_WAYLAND
 #include <gdk/gdkwayland.h>
 #endif
@@ -89,7 +92,7 @@ static void apply_ortho(guint mproj, float left, float right,
 
 static gboolean spice_egl_init_shaders(SpiceDisplay *display, GError **err)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     GLuint fs = 0, vs = 0, buf;
     GLint status, tex_loc, prog;
     gboolean success = FALSE;
@@ -179,7 +182,7 @@ end:
 G_GNUC_INTERNAL
 gboolean spice_egl_init(SpiceDisplay *display, GError **err)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     static const EGLint conf_att[] = {
         EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
         EGL_RENDERABLE_TYPE, EGL_OPENGL_BIT,
@@ -275,8 +278,8 @@ end:
 
     d->egl.context_ready = TRUE;
 
-    if (spice_display_get_gl_scanout(SPICE_DISPLAY_CHANNEL(d->display)) != NULL) {
-        SPICE_DEBUG("scanout present during egl init, updating widget");
+    if (spice_display_channel_get_gl_scanout(d->display) != NULL) {
+        DISPLAY_DEBUG(display, "scanout present during egl init, updating widget");
         spice_display_widget_gl_scanout(display);
         spice_display_widget_update_monitor_area(display);
     }
@@ -287,8 +290,11 @@ end:
 static gboolean
 gl_make_current(SpiceDisplay *display, GError **err)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
 
+    g_return_val_if_fail(d->egl.context_ready, FALSE);
+
+#ifdef GDK_WINDOWING_X11
     if (GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
         EGLBoolean success = eglMakeCurrent(d->egl.display,
                                             d->egl.surface,
@@ -301,11 +307,18 @@ gl_make_current(SpiceDisplay *display, GError **err)
             return FALSE;
         }
     }
+#endif
 #if GTK_CHECK_VERSION(3,16,0)
-    else {
+#ifdef GDK_WINDOWING_X11
+    else
+#endif
+    {
+        /* Ignore GLib's too-new warnings */
+        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         GtkWidget *area = gtk_stack_get_child_by_name(d->stack, "gl-area");
 
         gtk_gl_area_make_current(GTK_GL_AREA(area));
+        G_GNUC_END_IGNORE_DEPRECATIONS
     }
 #endif
 
@@ -315,7 +328,7 @@ gl_make_current(SpiceDisplay *display, GError **err)
 static gboolean spice_widget_init_egl_win(SpiceDisplay *display, GdkWindow *win,
                                           GError **err)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     EGLNativeWindowType native = 0;
 
     if (d->egl.surface)
@@ -352,7 +365,7 @@ static gboolean spice_widget_init_egl_win(SpiceDisplay *display, GdkWindow *win,
 G_GNUC_INTERNAL
 gboolean spice_egl_realize_display(SpiceDisplay *display, GdkWindow *win, GError **err)
 {
-    SPICE_DEBUG("egl realize");
+    DISPLAY_DEBUG(display, "egl realize");
     if (!spice_widget_init_egl_win(display, win, err))
         return FALSE;
 
@@ -365,9 +378,9 @@ gboolean spice_egl_realize_display(SpiceDisplay *display, GdkWindow *win, GError
 G_GNUC_INTERNAL
 void spice_egl_unrealize_display(SpiceDisplay *display)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
 
-    SPICE_DEBUG("egl unrealize %p", d->egl.surface);
+    DISPLAY_DEBUG(display, "egl unrealize %p", d->egl.surface);
 
     if (!gl_make_current(display, NULL))
         return;
@@ -397,6 +410,7 @@ void spice_egl_unrealize_display(SpiceDisplay *display)
         d->egl.prog = 0;
     }
 
+#ifdef GDK_WINDOWING_X11
     if (GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
         /* egl.surface && egl.ctx are only created on x11, see
            spice_egl_init() */
@@ -417,12 +431,13 @@ void spice_egl_unrealize_display(SpiceDisplay *display)
         /* do not call eglterminate() since egl may be used by
          * somebody else code */
     }
+#endif
 }
 
 G_GNUC_INTERNAL
 void spice_egl_resize_display(SpiceDisplay *display, int w, int h)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     int prog;
 
     if (!gl_make_current(display, NULL))
@@ -443,7 +458,7 @@ void spice_egl_resize_display(SpiceDisplay *display, int w, int h)
 static void
 draw_rect_from_arrays(SpiceDisplay *display, const void *verts, const void *tex)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
 
     glBindBuffer(GL_ARRAY_BUFFER, d->egl.vbuf_id);
 
@@ -515,7 +530,7 @@ client_draw_rect_tex(SpiceDisplay *display,
 G_GNUC_INTERNAL
 void spice_egl_cursor_set(SpiceDisplay *display)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     GdkPixbuf *image = d->mouse_pixbuf;
 
     g_return_if_fail(d->egl.enabled);
@@ -540,7 +555,7 @@ void spice_egl_cursor_set(SpiceDisplay *display)
 G_GNUC_INTERNAL
 void spice_egl_update_display(SpiceDisplay *display)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     double s;
     int x, y, w, h;
     gdouble tx, ty, tw, th;
@@ -555,10 +570,10 @@ void spice_egl_update_display(SpiceDisplay *display)
     glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
     glClear(GL_COLOR_BUFFER_BIT);
 
-    tx = ((float)d->area.x / (float)d->egl.scanout.width);
-    ty = ((float)d->area.y / (float)d->egl.scanout.height);
-    tw = ((float)d->area.width / (float)d->egl.scanout.width);
-    th = ((float)d->area.height / (float)d->egl.scanout.height);
+    tx = (gdouble) d->area.x / d->egl.scanout.width;
+    ty = (gdouble) d->area.y / d->egl.scanout.height;
+    tw = (gdouble) d->area.width / d->egl.scanout.width;
+    th = (gdouble) d->area.height / d->egl.scanout.height;
 
     /* convert to opengl coordinates, 0 is bottom, 1 is top. ty should
      * be the bottom of the area, since th is upward */
@@ -579,8 +594,8 @@ void spice_egl_update_display(SpiceDisplay *display)
         ty = 1 - ty;
         th = -1 * th;
     }
-    SPICE_DEBUG("update %f +%d+%d %dx%d +%f+%f %fx%f", s, x, y, w, h,
-                tx, ty, tw, th);
+    DISPLAY_DEBUG(display, "update %f +%d+%d %dx%d +%f+%f %fx%f", s, x, y, w, h,
+                  tx, ty, tw, th);
     glBindTexture(GL_TEXTURE_2D, d->egl.tex_id);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -614,10 +629,12 @@ void spice_egl_update_display(SpiceDisplay *display)
                              0, 0, 1, 1);
     }
 
+#ifdef GDK_WINDOWING_X11
     if (GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
         /* gtk+ does the swap with gtkglarea */
         eglSwapBuffers(d->egl.display, d->egl.surface);
     }
+#endif
 
     glUseProgram(prog);
 }
@@ -627,7 +644,7 @@ gboolean spice_egl_update_scanout(SpiceDisplay *display,
                                   const SpiceGlScanout *scanout,
                                   GError **err)
 {
-    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    SpiceDisplayPrivate *d = display->priv;
     EGLint attrs[13];
     guint32 format;
 
@@ -655,11 +672,11 @@ gboolean spice_egl_update_scanout(SpiceDisplay *display,
     attrs[10] = EGL_LINUX_DRM_FOURCC_EXT;
     attrs[11] = format;
     attrs[12] = EGL_NONE;
-    SPICE_DEBUG("fd:%d stride:%u y0:%d %ux%u format:0x%x (%c%c%c%c)",
-                scanout->fd, scanout->stride, scanout->y0top,
-                scanout->width, scanout->height, format,
-                (int)format & 0xff, (int)(format >> 8) & 0xff,
-                (int)(format >> 16) & 0xff, (int)format >> 24);
+    DISPLAY_DEBUG(display, "fd:%d stride:%u y0:%d %ux%u format:0x%x (%c%c%c%c)",
+                  scanout->fd, scanout->stride, scanout->y0top,
+                  scanout->width, scanout->height, format,
+                  (int)format & 0xff, (int)(format >> 8) & 0xff,
+                  (int)(format >> 16) & 0xff, (int)format >> 24);
 
     d->egl.image = eglCreateImageKHR(d->egl.display,
                                        EGL_NO_CONTEXT,
